#!/usr/bin/env python3

"""This tool is a FoLiA-aware wrapper around langid.py and provides languages identification on FoLiA documents"""

import sys
import os.path
import argparse
import glob
import shutil
import folia.main as folia
from langid.langid import LanguageIdentifier, model
from socket import getfqdn
from foliatools import VERSION as TOOLVERSION
from foliatools.foliavalidator import validate

LANG123 = { 'af': 'afr', 'am':'amh', 'an':'arg', 'ar':'ara', 'as':'asm', 'az':'aze', 'be':'bel', 'bg':'bul', 'bn':'ben', 'br':'bre', 'bs':'bos', 'ca':'cat', 'cs':'ces', 'cy':'cym', 'da':'dan', 'de':'deu', 'dz':'dzo', 'el':'ell', 'en':'eng', 'eo':'epo', 'es':'spa', 'et':'est', 'eu':'eus', 'fa':'fas', 'fi':'fin', 'fo':'fao', 'fr':'fra', 'ga':'gle', 'gl':'glg', 'gu':'guj', 'he':'heb', 'hi':'hin', 'hr':'hrv', 'ht':'hat', 'hu':'hun', 'hy':'hye', 'id':'ind', 'is':'isl', 'it':'ita', 'ja':'gap', 'jv':'jav', 'ka':'kat', 'kk':'kaz', 'km':'khm', 'kn': 'kan', 'ko':'kon', 'ku':'kur', 'ky':'kir', 'la':'lat', 'lb':'ltz', 'lo': 'lao', 'lt':'lit', 'lv': 'lav', 'mg':'mlg', 'mk':'mkd', 'ml':'mal', 'mn':'mon', 'mr':'mar', 'ms':'msa', 'mt':'mlt', 'nb':'nob', 'ne':'nep', 'nl':'nld', 'nn':'nno', 'no':'nor', 'oc':'oci', 'or':'ori', 'pa':'pan', 'pl':'pol', 'ps':'pus', 'pt':'por', 'qu':'que', 'ro':'ron', 'ru':'rus', 'rw':'kin', 'se':'sme', 'si':'sin', 'sk':'slk', 'sl':'slv', 'sq':'sqi', 'sr':'srp', 'sv':'swe', 'sw':'swa', 'ta':'tam', 'te':'tel', 'th':'tha', 'tl':'tgl', 'tr':'tur', 'ug':'uig', 'uk':'ukr', 'ur':'urd', 'vi':'vie', 'vo':'vol', 'wa':'wln', 'xh':'xho', 'zh':'zho', 'zu':'zul' }

LANG321 = { v: k for k, v in LANG123.items() }

LANGSET = "https://raw.githubusercontent.com/proycon/folia/master/setdefinitions/iso639_3.foliaset"

def main():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-v','-V','--version',help="Show version information", action='version', version="FoLiA-tools v" + TOOLVERSION + ", using FoLiA v" + folia.FOLIAVERSION + " with library FoLiApy v" + folia.LIBVERSION, default=False)
    parser.add_argument('-n', '--dryrun',help="Dry run, do not write files (outputs to stdout)", action='store_true', default=False)
    parser.add_argument('-E','--extension', type=str,help="Extension", action='store',default="xml",required=False)
    parser.add_argument('-l','--languages', type=str,help="Constrain possible languages (comma separated list of ISO-639-3 codes)", action='store',required=False)
    parser.add_argument('-t','--types', type=str,help="Constrain to the following structural elements (comma separated list of FoLiA structural element tags, e.g. 's' for sentence, 'p' for paragraph). Defaults to annotating EVERY structural element.", action='store',required=False)
    parser.add_argument('-T','--textclass', type=str,help="Use the specified textclass (default: current)", action='store',required=False, default="current")
    parser.add_argument('-c','--confidence', type=float, help="Confidence threshold, anything below this will be pruned or set to the fallback language (if set)", action='store')
    parser.add_argument('-F','--fallback', type=str, help="Fallback language, everything not meeting the confidence threshold will be set to this language (with a low confidence value of 0.1", action='store')
    parser.add_argument('--verbose',help="Output verbose information", action='store_true')
    parser.add_argument('files', nargs='+', help='Input files')
    args = parser.parse_args()

    success = process(*args.files, **args.__dict__)
    sys.exit(0 if success else 1)

def processdoc(doc, **kwargs):
    identifier = LanguageIdentifier.from_modelstring(model, norm_probs=True)
    if 'languages' in kwargs and kwargs['languages']:
        identifier.set_languages(kwargs['languages'].split(','))
    if 'confidence' in kwargs:
        confidencethreshold = kwargs['confidence']
    else:
        confidencethreshold = 0.0
    if 'types' in kwargs and kwargs['types']:
        types = tuple([ folia.XML2CLASS[t] for t in kwargs['types'].split(',') ])
    else:
        types = tuple([ folia.AbstractStructureElement ])
    doc.processor = folia.Processor.create(name="folialangid", version=TOOLVERSION, src="https://github.com/proycon/foliatools")
    doc.declare(folia.AnnotationType.LANG, set=LANGSET)
    for e in doc.data[0].select(types):
        try:
            text = e.text(cls=kwargs.get("textclass","current"))
        except folia.NoSuchText:
            continue #ok, no text, fine with me, nothing to do then
        if text.strip():
            lang, confidence = identifier.classify(text)
            if kwargs.get('verbose'):
                print(repr(e), lang, confidence,file=sys.stderr)
            if e.accepts(folia.LangAnnotation, raiseexceptions=False):
                if confidence > confidencethreshold:
                    e.append(folia.LangAnnotation, set=LANGSET,cls=LANG123[lang], confidence=confidence)
                elif 'fallback' in kwargs:
                    e.append(folia.LangAnnotation, set=LANGSET,cls=kwargs['fallback'], confidence=0.1)

def process(*files, **kwargs):
    success = True
    for file in files:
        if os.path.isdir(file):
            r = process(list(glob.glob(os.path.join(file, "*." + kwargs['extension'] )) ), **kwargs)
            if r != 0:
                success = False
        elif os.path.isfile(file):
            if kwargs.get('verbose'): print("Loading ", file ,file=sys.stderr)
            doc = folia.Document(file=file,autodeclare=True)
            if kwargs.get('verbose'): print("Processing ", file ,file=sys.stderr)
            processdoc(doc, **kwargs)
            if kwargs.get('dryrun'):
                print(doc.xmlstring())
            else:
                if kwargs.get('verbose'): print("Saving ", file ,file=sys.stderr)
                doc.save(doc.filename + ".upgraded")
    return success

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
This is a benchmarking tool for the foliapy library.
"""

from __future__ import print_function, unicode_literals, division, absolute_import

import sys
import os
import argparse
import time
import gc
import resource
from foliatools import VERSION as TOOLVERSION
import folia.main as folia

ansicolors = {"red":31,"green":32,"yellow":33,"blue":34,"magenta":35, "bold":1 }
def colorf(color, x):
    return "\x1B[" + str(ansicolors[str(color)]) + "m" + x + "\x1B[0m"


def getmem():
    rusage_denom = 1024.
    if sys.platform == 'darwin':
        # ... it seems that in OSX the output is different units ...
        rusage_denom = rusage_denom * rusage_denom
    mem = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / rusage_denom
    return mem

def begin():
    gc.collect()
    return time.time(), getmem()

def end(data):
    begintime, beginmem = data
    d = (time.time() - begintime)
    d = d * 1000
    memd = getmem() - beginmem
    gc.collect()
    return d, memd


def run_test(test_id, file):
    if test_id == 'parse':
        title = "Parse XML from file into full memory representation"
        begintime, beginmem = begin()
        doc = folia.Document(file=file)
        return begintime, beginmem, title , doc
    elif test_id == 'serialise':
        title = "Serialise to XML"
        doc = folia.Document(file=file)
        begintime, beginmem = begin()
        out = doc.xmlstring()
        return begintime, beginmem, title , out
    elif test_id == 'text':
        title = "Serialise to text"
        doc = folia.Document(file=file)
        begintime, beginmem = begin()
        out = doc.text()
        return begintime, beginmem, title , out
    elif test_id == 'select':
        title = "Select all words"
        doc = folia.Document(file=file)
        begintime, beginmem = begin()
        for word in doc.select(folia.Word):
            pass
        return begintime, beginmem, title, None
    raise KeyError("No such test: " + test_id)

def test(test_id,file, args):
    times = []
    mems = []
    title = ""
    for i in range(0,args.iterations):
        begintime, beginmem, title, doc = run_test(test_id,file)
        t, mem  = end((begintime, beginmem))
        times.append(t)
        mems.append(mem)
    t = sum(times) / float(len(times))
    mem = sum(mems) / float(len(mems))
    print(file +  " - [" + test_id+ "] " + title + ": " + colorf('yellow',str(round(t,3))+ 'ms') + " -- Memory: " + colorf('green',str(round(mem,2))+ ' MB') + "\n")


def main():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-v','-V','--version',help="Show version information", action='version', version="FoLiA-tools v" + TOOLVERSION + ", using FoLiA v" + folia.FOLIAVERSION + " with library FoLiApy v" + folia.LIBVERSION, default=False)
    parser.add_argument('-i','--iterations', type=int,help="Number of iterations to run and average over", action='store',default=1,required=False)
    parser.add_argument('-t','--tests', type=str,help="Comma separated list of test IDs", action='store',default="parse,serialise,text,words",required=False)
    parser.add_argument('files', nargs='*', help='Files (and/or directories) to validate')
    args = parser.parse_args()

    for file in args.files:
        for test_id in args.tests.split(","):
            test(test_id, file, args)


if __name__ == '__main__':
    main()





def makencname(s):
    s = s.replace(" ","_")
    s = s.replace(":","_")
    if s and not s[0].isalpha() and s[0] != '_':
        s = "_" + s
    return s

def set_metadata(doc, **kwargs):
    for key, value in kwargs.items():
        doc.metadata[key] = value




#!/usr/bin/env python
#-*- coding:utf-8 -*-


from __future__ import print_function, unicode_literals, division, absolute_import

import getopt
import io
import sys
import os
import mistune
try:
    from pynlpl.formats import folia
except:
    print("ERROR: pynlpl not found, please obtain PyNLPL from the Python Package Manager ($ sudo easy_install pynlpl) or directly from github: $ git clone git://github.com/proycon/pynlpl.git", file=sys.stderr)
    sys.exit(2)

def usage():
    print("markdown2folia",file=sys.stderr)
    print("  by Maarten van Gompel (proycon)",file=sys.stderr)
    print("  Tilburg University / Radboud University Nijmegen",file=sys.stderr)
    print("  2014 - Licensed under GPLv3",file=sys.stderr)
    print("",file=sys.stderr)
    print("This conversion script converts a markdown document to FoLiA XML",file=sys.stderr)
    print("",file=sys.stderr)
    print("Usage: markdown2folia [options] file",file=sys.stderr)
    print("",file=sys.stderr)
    print("Input parameters",file=sys.stderr)
    print("  -e [filename]                Encoding of the markdown file (default: utf-8",file=sys.stderr)
    print("Parameters for output:",file=sys.stderr)
    print("  -o [filename]                Output to a single file (instead of default stdout)",file=sys.stderr)



class MD2FoLiARenderer(mistune.Renderer):
    def paragraph(self, text):
        return '<p><t>' + text + '</t></p>'

    def block_code(self, code, lang):
        if lang:
            return '<gap><desc>code (' + lang + ')</desc><content>' + code + '</content></gap>'
        else:
            return '<gap><desc>code</desc><content>' + code + '</content></gap>'

def process(filename, encoding, outputfile = None):
    print("Converting " + filename, file=sys.stderr)
    md = mistune.Markdown(renderer=MD2FoLiARenderer())
    with io.open(filename,'r', encoding=encoding) as f:
        if outputfile:
            with io.open(outputfile,'w',encoding='utf-8') as fout:
                fout.write(md.render(f.read()) + "\n")
        else:
            print(md.render(f.read()))


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "o:e:", ["help"])
    except getopt.GetoptError as err:
        print(str(err), file=sys.stderr)
        usage()
        sys.exit(2)


    outputfile = None
    encoding= 'utf-8'


    for o, a in opts:
        if o == '-h' or o == '--help':
            usage()
            sys.exit(0)
        elif o == '-o':
            outputfile = a
        elif o == '-e':
            encoding = a
        else:
            raise Exception("No such option: " + o)




    if args:
        x = args[0]
        if os.path.isfile(x):
            process(x, encoding, outputfile)
        else:
            print("ERROR: File not found: " + x,file=sys.stderr)
            sys.exit(3)
    else:
        print("ERROR: Nothing to do, specify a file",file=sys.stderr)

    if outputfile: outputfile.close()

if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf8 -*-

import os
import io
from setuptools import setup


def read(fname):
    return io.open(os.path.join(os.path.dirname(__file__), fname),'r',encoding='utf-8').read()

setup(
    name = "FoLiA-tools",
    version = "2.4.3", #also change in __init__.py
    author = "Maarten van Gompel",
    author_email = "proycon@anaproy.nl",
    description = ("FoLiA-tools contains various Python-based command line tools for working with FoLiA XML (Format for Linguistic Annotation)"),
    license = "GPL",
    keywords = "nlp computational_linguistics search linguistics toolkit folia pynlpl format_for_linguistic_annotation",
    url = "https://proycon.github.io/folia",
    packages=['foliatools'],
    long_description=read('README.rst'),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Text Processing :: Linguistic",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: POSIX",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
    entry_points = {
        'console_scripts': [
            'folia2txt = foliatools.folia2txt:main',
            'folia2annotatedtxt = foliatools.folia2annotatedtxt:main',
            'foliafreqlist = foliatools.foliafreqlist:main',
            'foliavalidator = foliatools.foliavalidator:main',
            'foliamerge = foliatools.foliamerge:main',
            'foliaerase = foliatools.foliaerase:main',
            'folia2columns = foliatools.folia2columns:main',
            'folia2dcoi = foliatools.folia2dcoi:main',
            'folia2html = foliatools.folia2html:main',
            'folia2salt = foliatools.folia2salt:main',
            'foliaquery = foliatools.foliaquery:main',
            'foliaquery1 = foliatools.foliaquery1:main', #old version
            'foliatextcontent = foliatools.foliatextcontent:main',
            'dcoi2folia = foliatools.dcoi2folia:main',
            'conllu2folia = foliatools.conllu2folia:main',
            'rst2folia = foliatools.rst2folia:main',
            'txt2folia = foliatools.txt2folia:main',
            'tei2folia = foliatools.tei2folia:main',
            'foliacat = foliatools.foliacat:main',
            'folia2rst = foliatools.folia2rst:main',
            'foliacorrect = foliatools.foliacorrect:main',
            'foliacount = foliatools.foliacount:main',
            'foliaid = foliatools.foliaid:main',
            'foliaspec = foliatools.foliaspec:main',
            'foliaspec2json = foliatools.foliaspec2json:main',
            'alpino2folia = foliatools.alpino2folia:main',
            'foliatree = foliatools.foliatree:main',
            'foliasetdefinition = foliatools.foliasetdefinition:main',
            'foliaeval = foliatools.foliaeval:main',
            'foliaupgrade = foliatools.foliaupgrade:main',
            'folialangid = foliatools.folialangid:main',
            'foliabench = foliatools.foliabench:main',
            'foliasplit = foliatools.foliasplit:main',
        ]
    },
    #include_package_data=True,
    package_data = {'foliatools': ['*.xsl']},
    install_requires=['folia >= 2.4.1', 'lxml >= 2.2','docutils', 'pyyaml', 'langid','conllu', 'requests']
)
